package sun.security.krb5;

import sun.security.krb5.internal.KrbApErrException;
import sun.security.krb5.internal.KdcErrException;
import sun.security.krb5.internal.KRBSafeBody;
import sun.security.krb5.internal.KRBSafe;
import java.io.IOException;
import sun.security.krb5.internal.HostAddress;
import sun.security.krb5.internal.SeqNumber;
import sun.security.krb5.internal.KerberosTime;

class KrbSafe extends KrbAppMessage
{
    private byte[] obuf;
    private byte[] userData;
    
    public KrbSafe(byte[] array, Credentials credentials, EncryptionKey encryptionKey, KerberosTime kerberosTime, SeqNumber seqNumber, HostAddress hostAddress, HostAddress hostAddress2) throws KrbException, IOException {
        EncryptionKey key;
        if (encryptionKey != null) {
            key = encryptionKey;
        }
        else {
            key = credentials.key;
        }
        this.obuf = this.mk_safe(array, key, kerberosTime, seqNumber, hostAddress, hostAddress2);
    }
    
    public KrbSafe(byte[] array, Credentials credentials, EncryptionKey encryptionKey, SeqNumber seqNumber, HostAddress hostAddress, HostAddress hostAddress2, boolean b, boolean b2) throws KrbException, IOException {
        KRBSafe krbSafe = new KRBSafe(array);
        EncryptionKey key;
        if (encryptionKey != null) {
            key = encryptionKey;
        }
        else {
            key = credentials.key;
        }
        this.userData = this.rd_safe(krbSafe, key, seqNumber, hostAddress, hostAddress2, b, b2, credentials.client);
    }
    
    public byte[] getMessage() {
        return this.obuf;
    }
    
    public byte[] getData() {
        return this.userData;
    }
    
    private byte[] mk_safe(byte[] array, EncryptionKey encryptionKey, KerberosTime kerberosTime, SeqNumber seqNumber, HostAddress hostAddress, HostAddress hostAddress2) throws Asn1Exception, IOException, KdcErrException, KrbApErrException, KrbCryptoException {
        Integer n = null;
        Object o = null;
        if (kerberosTime != null) {
            n = new Integer(kerberosTime.getMicroSeconds());
        }
        if (seqNumber != null) {
            o = new Integer(seqNumber.current());
            seqNumber.step();
        }
        KRBSafeBody o2 = new KRBSafeBody(array, kerberosTime, n, o, hostAddress, hostAddress2);
        KRBSafe o3 = new KRBSafe(o2, new Checksum(Checksum.SAFECKSUMTYPE_DEFAULT, o2.asn1Encode(), encryptionKey, 15));
        o3.asn1Encode();
        return o3.asn1Encode();
    }
    
    private byte[] rd_safe(KRBSafe krbSafe, EncryptionKey encryptionKey, SeqNumber seqNumber, HostAddress hostAddress, HostAddress hostAddress2, boolean b, boolean b2, PrincipalName principalName) throws Asn1Exception, KdcErrException, KrbApErrException, IOException, KrbCryptoException {
        if (!krbSafe.cksum.verifyKeyedChecksum(krbSafe.safeBody.asn1Encode(), encryptionKey, 15)) {
            throw new KrbApErrException(41);
        }
        this.check(krbSafe.safeBody.timestamp, krbSafe.safeBody.usec, krbSafe.safeBody.seqNumber, krbSafe.safeBody.sAddress, krbSafe.safeBody.rAddress, seqNumber, hostAddress, hostAddress2, b, b2, principalName);
        return krbSafe.safeBody.userData;
    }
}
